--
-- TrueMirror
--
-- Seiten Spiegel (Links/Rechts)
--
-- by Mogli
--

TrueMirror = {};

local TrueMirror_directory = g_currentModDirectory;

------------------------------------------------------------------------
-- prerequisitesPresent
------------------------------------------------------------------------
function TrueMirror.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Motorized, specializations);
end;

------------------------------------------------------------------------
-- load
------------------------------------------------------------------------
function TrueMirror:load()

  self.mouseEvent = Utils.overwrittenFunction( self.mouseEvent, TrueMirror.newMouseEvent );				

	self.tmShow = false;
	self.tmGuiActive = false;
	self.tmLinked = false;
	self.tmLastLinked = false;
	self.tmRootNode = getChild(Utils.loadSharedI3DFile("TrueMirror2.i3d", TrueMirror_directory),"true_mirror");
	self.tmTimeout = 0;
	self.tmLeft = getChild(self.tmRootNode,"left");
	self.tmRight = getChild(self.tmRootNode,"right");
	self.tmTx, self.tmTy, self.tmTz = getTranslation(self.tmLeft);
	self.tmRx, self.tmRy, self.tmRz = getRotation(self.tmLeft);
	self.tmSx, self.tmSy, self.tmSz = getScale(self.tmLeft);
	self.tmDx = 0;
	self.tmDy = 0;
	self.tmDz = 0;
	self.tmDu = 0;
	self.tmDv = 0;
	self.tmDs = 1;
	self.tmBtnPosX = 0.28;
	self.tmBtnPosY = 0.60; 
	self.tmOverlay = Overlay:new("TrueMirrorHud", Utils.getFilename("Hud.dds", TrueMirror_directory), 0.2625, 0.0108, 0.215, 0.215);
	self.helpPanelActive = false;	
	
	self.tmButtons = {};
	
	TrueMirror.addButton(self,"off.dds", "on.dds", TrueMirror.onShow, TrueMirror.evalShow, 0.00,0.40, "TrueMirror_on", "TrueMirror_off" );
	
	TrueMirror.addButton(self, "up.dds"       , nil, TrueMirror.onMoveUp  , nil, 0.04,0.40, "TrueMirror_mup");
	TrueMirror.addButton(self, "rot_up.dds"   , nil, TrueMirror.onRotUp   , nil, 0.08,0.40, "TrueMirror_rup");
	TrueMirror.addButton(self, "forward.dds"  , nil, TrueMirror.onMoveFwd , nil, 0.12,0.40, "TrueMirror_fwd");
	TrueMirror.addButton(self, "plus.dds"     , nil, TrueMirror.onBigger  , nil, 0.16,0.40, "TrueMirror_big");	
                                                                                        
	TrueMirror.addButton(self, "bigger.dds"   , nil, TrueMirror.onMoveOut , nil, 0.00,0.45, "TrueMirror_out");
	TrueMirror.addButton(self, "rot_left.dds" , nil, TrueMirror.onRotLeft , nil, 0.04,0.45, "TrueMirror_left");
	TrueMirror.addButton(self, "rot_right.dds", nil, TrueMirror.onRotRight, nil, 0.12,0.45, "TrueMirror_right");
	TrueMirror.addButton(self, "smaller.dds"  , nil, TrueMirror.onMoveIn  , nil, 0.16,0.45, "TrueMirror_in");	
                                                                                       
	TrueMirror.addButton(self, "down.dds"     , nil, TrueMirror.onMoveDown, nil, 0.04,0.50, "TrueMirror_mdown");
	TrueMirror.addButton(self, "rot_down.dds" , nil, TrueMirror.onRotDown , nil, 0.08,0.50, "TrueMirror_rdown");
	TrueMirror.addButton(self, "backward.dds" , nil, TrueMirror.onMoveBack, nil, 0.12,0.50, "TrueMirror_back");
	TrueMirror.addButton(self, "minus.dds"    , nil, TrueMirror.onSmaller , nil, 0.16,0.50, "TrueMirror_small");	
end;                                                          

------------------------------------------------------------------------
-- addButton
------------------------------------------------------------------------
function TrueMirror:addButton(imgEnabled, imgDisabled, cbOnClick, cbVisible, x,y, textEnabled, textDisabled, textCallback, width,height)
	local x1 = self.tmBtnPosX + x;
	local y2 = self.tmBtnPosY - y;
	local x2 = x1+0.03;
	if width ~= nil then
		local x2 = x1+abs(width);
	end;
	local y1 = y2-0.04;
	if height ~= nil then
		local y1 = y2-abs(height);
	end;
	local overlay = Overlay:new(nil, Utils.getFilename( imgEnabled, TrueMirror_directory), x1,y1,x2-x1,y2-y1);
	local overlay2 = nil;
	if imgDisabled == nil then
		overlay2 = Overlay:new(nil, Utils.getFilename( "empty.dds", TrueMirror_directory), x1,y1,x2-x1,y2-y1);
	else
		overlay2 = Overlay:new(nil, Utils.getFilename( imgDisabled, TrueMirror_directory), x1,y1,x2-x1,y2-y1);
  end;
  local button = {enabled=true, ovEnabled=overlay, ovDisabled=overlay2, onClick=cbOnClick, onVisible=cbVisible, twoState=(imgDisabled ~= nil), rect={x1,y1,x2,y2}, text1 = textEnabled, text2 = textDisabled, textcb = textCallback };
  table.insert(self.tmButtons, button);
  return button;
end;

------------------------------------------------------------------------
-- newMouseEvent
------------------------------------------------------------------------
function TrueMirror:newMouseEvent(superFunc, posX, posY, isDown, isUp, button)
	if self.tmGuiActive then
		local x = InputBinding.mouseMovementX;
		local y = InputBinding.mouseMovementY;
		InputBinding.mouseMovementX = 0;
		InputBinding.mouseMovementY = 0;
		superFunc(self,posX, posY, isDown, isUp, button);
		InputBinding.mouseMovementX = x;
		InputBinding.mouseMovementY = y;
	else
		superFunc(self,posX, posY, isDown, isUp, button);
	end;
end;

local function tmGetText(id)
	if id == nil then
		return "nil";
	end;
	
	local text = g_i18n:getText( id ); --g_i18n.globalI18N.texts[id];
	if text == nil or text == "" then
		return id;
	end;
	
	return text;
end;

------------------------------------------------------------------------
-- mouseEvent
------------------------------------------------------------------------
function TrueMirror:mouseEvent(posX, posY, isDown, isUp, button)

	self.tmTooltip = nil;
	local textID, textCB;
	if self.tmGuiActive then
		for _,overlayButton in pairs(self.tmButtons) do
			if overlayButton.rect[1] <= posX and posX <= overlayButton.rect[3] and overlayButton.rect[2] <= posY and posY <= overlayButton.rect[4] then
				if overlayButton.onClick ~= nil and isDown and button == 1 then
					if overlayButton.enabled then
						if overlayButton.twoState ~= nil then
							overlayButton.onClick(self, true);
						else
							overlayButton.onClick(self);
						end;
					elseif overlayButton.twoState then
						overlayButton.onClick(self, false);
					end;
					--TrueMirror.sendParameters(self);
				end
				if  overlayButton.text1 ~= nil 
						and ( overlayButton.enabled 
							or not overlayButton.twoState 
							or overlayButton.text2 == nil ) then
					textID = overlayButton.text1;
				elseif overlayButton.text2 ~= nil then
					textID = overlayButton.text2;
				end;
				textCB = overlayButton.textcb;
				break;				
			end;
		end;
	end;
	
	if textID ~= nil then
		self.tmTooltip = tmGetText( textID );
		if textCB ~= nil then 
			self.tmTooltip = textCB(self,self.tmTooltip) 
		end
	end
end

------------------------------------------------------------------------
-- renderButtons
------------------------------------------------------------------------
function TrueMirror.renderButtons(self)
  for _,button in pairs(self.tmButtons) do
    if button.onVisible ~= nil then
			button.enabled = button.onVisible(self);
		end;
		if button.enabled then
			if button.ovEnabled ~= nil then
				button.ovEnabled:render();
			end;
		else
			if button.ovDisabled ~= nil then
				button.ovDisabled:render();
			end;
		end;
  end;	
end;

------------------------------------------------------------------------
-- mouse event callbacks
------------------------------------------------------------------------
function TrueMirror.showGui(self,on)
  self.tmGuiActive = on;
  g_mouseControlsHelp.active = not on;
	InputBinding.setShowMouseCursor(on);		
end;

function TrueMirror:evalShow()
	return not self.tmShow
end

function TrueMirror:onShow(enabled)
	self.tmShow = enabled
end

function TrueMirror:onMoveUp() 
	self.tmDv = self.tmDv+0.05;
	TrueMirror.moveMirror(self); 
end
function TrueMirror:onRotUp() 
	self.tmDu = self.tmDu+0.02;
	TrueMirror.moveMirror(self); 
end
function TrueMirror:onMoveFwd() 
	self.tmDz = self.tmDz+0.05;
	TrueMirror.moveMirror(self); 
end
function TrueMirror:onBigger() 
	self.tmDs = self.tmDs*1.1;
	TrueMirror.moveMirror(self); 
end
function TrueMirror:onMoveOut() 
	self.tmDx = self.tmDx-0.05;
	TrueMirror.moveMirror(self); 
end
function TrueMirror:onRotLeft() 
	self.tmDy = self.tmDy+0.02;
	TrueMirror.moveMirror(self); 
end
function TrueMirror:onRotRight() 
	self.tmDy = self.tmDy-0.02;
	TrueMirror.moveMirror(self); 
end
function TrueMirror:onMoveIn() 
	self.tmDx = self.tmDx+0.05;
	TrueMirror.moveMirror(self); 
end
function TrueMirror:onMoveDown() 
	self.tmDv = self.tmDv-0.05;
	TrueMirror.moveMirror(self); 
end
function TrueMirror:onRotDown() 
	self.tmDu = self.tmDu-0.02;
	TrueMirror.moveMirror(self); 
end
function TrueMirror:onMoveBack() 
	self.tmDz = self.tmDz-0.05;
	TrueMirror.moveMirror(self); 
end
function TrueMirror:onSmaller() 
	self.tmDs = self.tmDs/1.1;
	TrueMirror.moveMirror(self); 
end

------------------------------------------------------------------------
-- keyEvent
------------------------------------------------------------------------
function TrueMirror:keyEvent(unicode, sym, modifier, isDown)
end;

------------------------------------------------------------------------
-- update
------------------------------------------------------------------------
function TrueMirror:update()

	local moveMirror = false;

	if self:getIsActiveForInput(false) then
		if self.tmTimeout <= 0 and InputBinding.hasEvent(InputBinding.TrueMirror) then
    	    --self.tmTimeout = 100;
			TrueMirror.showGui(self,not self.tmGuiActive)
	  end;
		if self.tmLastLinked then
			if InputBinding.hasEvent(InputBinding.TrueMirror_fwd) then
				moveMirror = true;
				self.tmDz = self.tmDz+0.05;
			end;
			if InputBinding.hasEvent(InputBinding.TrueMirror_back) then
				moveMirror = true;
				self.tmDz = self.tmDz-0.05;
			end;
			if InputBinding.hasEvent(InputBinding.TrueMirror_mup) then
				moveMirror = true;
				self.tmDv = self.tmDv+0.05;
			end;
			if InputBinding.hasEvent(InputBinding.TrueMirror_mdown) then
				moveMirror = true;
				self.tmDv = self.tmDv-0.05;
			end;
			if InputBinding.hasEvent(InputBinding.TrueMirror_in) then
				moveMirror = true;
				self.tmDx = self.tmDx+0.05;
			end;
			if InputBinding.hasEvent(InputBinding.TrueMirror_out) then
				moveMirror = true;
				self.tmDx = self.tmDx-0.05;
			end;
			if InputBinding.hasEvent(InputBinding.TrueMirror_right) then
				moveMirror = true;
				self.tmDy = self.tmDy+0.02;
			end;
			if InputBinding.hasEvent(InputBinding.TrueMirror_left) then
				moveMirror = true;
				self.tmDy = self.tmDy-0.02;
			end;
			if InputBinding.hasEvent(InputBinding.TrueMirror_rup) then
				moveMirror = true;
				self.tmDu = self.tmDu+0.02;
			end;
			if InputBinding.hasEvent(InputBinding.TrueMirror_rdown) then
				moveMirror = true;
				self.tmDu = self.tmDu-0.02;
			end;
			if InputBinding.hasEvent(InputBinding.TrueMirror_big) then
				moveMirror = true;
				self.tmDs = self.tmDs*1.1;
			end;
			if InputBinding.hasEvent(InputBinding.TrueMirror_small) then
				moveMirror = true;
				self.tmDs = self.tmDs/1.1;
			end;
		end;
	end;
	
	if self.tmTimeout > 0 then self.tmTimeout = self.tmTimeout - 1; end;
	if self.tmDs < 0.1 then self.tmDs = 1; end;
	
	self.tmLinked = self.tmShow and ( self.characterNode ~= nil );
	
	if self.tmLastLinked ~= self.tmLinked then		
		if self.tmLinked then
			--link(self.characterNode,self.tmRootNode);
			--local x, y, z = getWorldRotation(self.tmRootNode);
			--setRotation(self.tmRootNode,-x,-y,-z);
			link(getParent(self.characterNode),self.tmRootNode);
			local x, y, z = getTranslation(self.characterNode);
			setTranslation(self.tmRootNode,x,y,z);
			self.tmLastLinked = true;
			moveMirror = true;
		elseif self.tmRootNode ~= 0 then
			unlink(self.tmRootNode);
			self.tmLastLinked = false;
		end;
	end;
	if moveMirror then
		TrueMirror.moveMirror(self);
	end;
end;

function TrueMirror:moveMirror()
	setTranslation(self.tmLeft,self.tmTx+self.tmDx,self.tmTy+self.tmDv,self.tmTz+self.tmDz);
	setTranslation(self.tmRight,-self.tmTx-self.tmDx,self.tmTy+self.tmDv,self.tmTz+self.tmDz);
	setRotation(self.tmLeft,self.tmRx+self.tmDu,self.tmRy+self.tmDy,self.tmRz);
	setRotation(self.tmRight,self.tmRx+self.tmDu,-self.tmRy-self.tmDy,self.tmRz);		
	setScale(self.tmLeft,self.tmSx*self.tmDs,self.tmSy*self.tmDs,self.tmSz*self.tmDs);		
	setScale(self.tmRight,self.tmSx*self.tmDs,self.tmSy*self.tmDs,self.tmSz*self.tmDs);		
end;
	
function TrueMirror:draw()
  g_currentMission:addHelpButtonText(Steerable.TrueMirror, InputBinding.TrueMirror);
	if self.tmGuiActive then
		self.tmOverlay:render();
		TrueMirror.renderButtons(self);
		if self.tmTooltip ~= nil and self.tmTooltip ~= "" then
			renderText(self.tmBtnPosX, 0.02, 0.021,self.tmTooltip);
		end
	end
end;

function TrueMirror:delete()
end;

function TrueMirror:onLeave()
end;

function TrueMirror:getSaveAttributesAndNodes(nodeIdent)
	local attributes = "";
	if     self.tmShow 
	    or self.tmDx ~= 0
	    or self.tmDy ~= 0
	    or self.tmDz ~= 0
	    or self.tmDu ~= 0
	    or self.tmDv ~= 0
	    or self.tmDs ~= 1
	  then
		if self.tmShow then
			attributes = 'trueMirror="1" ';
		else
			attributes = 'trueMirror="0" ';
		end;
		attributes = attributes..'trueMirrorDx="'..self.tmDx..'" ';		
		attributes = attributes..'trueMirrorDy="'..self.tmDy..'" ';
		attributes = attributes..'trueMirrorDz="'..self.tmDz..'" ';
		attributes = attributes..'trueMirrorDu="'..self.tmDu..'" ';
		attributes = attributes..'trueMirrorDv="'..self.tmDv..'" ';
		attributes = attributes..'trueMirrorDs="'..self.tmDs..'" ';
	end
	return attributes
end;


function TrueMirror:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	local l = getXMLInt(xmlFile, key.."#trueMirror");
	if l == 1 or l==0 then
		self.tmShow = l==1;			
		local x = getXMLFloat(xmlFile, key.."#trueMirrorDx"); if x ~= nil then self.tmDx = x; end;
		local y = getXMLFloat(xmlFile, key.."#trueMirrorDy"); if y ~= nil then self.tmDy = y; end;
		local z = getXMLFloat(xmlFile, key.."#trueMirrorDz"); if z ~= nil then self.tmDz = z; end;
		local u = getXMLFloat(xmlFile, key.."#trueMirrorDu"); if u ~= nil then self.tmDu = u; end;
		local v = getXMLFloat(xmlFile, key.."#trueMirrorDv"); if v ~= nil then self.tmDv = v; end;
		local s = getXMLFloat(xmlFile, key.."#trueMirrorDs"); if s ~= nil then self.tmDs = s; end;
	end;	
	return BaseMission.VEHICLE_LOAD_OK;
end
